/***************************************************************************
 *   Copyright (C) 2005 by Giovanni Racciu                                 *
 *   gracciu@xystumnet.com                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <q3socket.h>
#include <qapplication.h>
#include <q3vbox.h>
#include <q3hbox.h>
#include <q3textview.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <q3textstream.h>
#include <q3filedialog.h>
#include <qmessagebox.h>
#include <q3progressbar.h>
#include <qstringlist.h>
//Added by qt3to4:
#include <QPixmap>
#define QTCLAMAVVER "Qt ClamAV client v0.02"
#define max_stream_size 10000000 //this value defines the maximum files size to be scanned and should be equal to the StreamMaxLength parameter in the clamd.conf file.
//#define debug //uncomment this line for verbose output

class Client : public Q3VBox
{
    Q_OBJECT
    
public:
    Client( const QString &host, const QString &port) 
    {
	QPixmap clamIcon;
	// Basic GUI layout
	infoText = new Q3TextView( this );
	streamprogress = new Q3ProgressBar(this);
	clamav_stream_port = 0;
	ClamAV_host = host;
	clamIcon = QPixmap( "clamav.xpm" );
    	QPushButton *scan = new QPushButton(clamIcon , "Scan" , this);
	
	Q3HBox *hb = new Q3HBox( this );
	QPushButton *about = new QPushButton( "About" , hb );
	QPushButton *aboutQt = new QPushButton( "About Qt" , hb );
	QPushButton *quit = new QPushButton( "Quit" , hb);
	
	sscanf(port, "%hu", &clamav_main_port );
	connect( quit, SIGNAL(clicked()), qApp, SLOT(quit()) );
	connect( scan, SIGNAL(clicked()), SLOT(ClamAVScan()) );
	connect( aboutQt, SIGNAL(clicked()), SLOT(aboutQt()) );
	connect( about, SIGNAL(clicked()), SLOT(about()) );

	// create the socket and connect various of its signals
	socket = new Q3Socket( this );
	connect( socket, SIGNAL(connected()), SLOT(socketConnected()) );
	connect( socket, SIGNAL(connectionClosed()), SLOT(socketConnectionClosed()) );
	connect( socket, SIGNAL(readyRead()), SLOT(socketReadyRead()) );
	connect( socket, SIGNAL(error(int)), SLOT(socketError(int)) );

	// create the STREAM socket and connect various of its signals
        stream_socket = new Q3Socket( this );
	connect( stream_socket , SIGNAL(connected()), SLOT(Stream_socketConnected()) );
	connect( stream_socket , SIGNAL(connectionClosed()), SLOT(Stream_socketConnectionClosed()) );
	connect( stream_socket , SIGNAL(readyRead()), SLOT(socketReadyRead()) );
	connect( stream_socket , SIGNAL(error(int)), SLOT(socketError(int)) );
	
	// connect to the server
	strcpy (&Output_String[0], "Trying to connect to the server ");
	strcat (Output_String, ClamAV_host);
	strcat (Output_String, " on port ");
	strcat (Output_String, port);
       	infoText->append( Output_String );	
	socket->connectToHost( ClamAV_host, clamav_main_port  );
	//Get ClamAv Version
	sendStringToServer("VERSION");
    }

    ~Client()
    {
    }

private slots:

    void about ()
    {
    QMessageBox::about( this, QTCLAMAVVER,
			"This is a simple client for the ClamAV server\nCopyright (C) 2005 by Giovanni Racciu\ngracciu@xystumnet.com\nThis program is distributed under the terms of the GPL v2.\nThis program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;\nwithout even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\nSee the GNU General Public License for more details.");
    }
    void aboutQt ()
    {
    QMessageBox::aboutQt(this, QTCLAMAVVER);
    }
    
    void ClamAVScan ()
    {
    QStringList filelisttoscan = Q3FileDialog::getOpenFileNames( QString::null, QString::null,
					       this, QTCLAMAVVER, "Qt Clam AV Client - Select file(s) to be scanned");
    files_to_scan = filelisttoscan;
    
    it = files_to_scan.begin(); 
    scanninginprogress = 1;
    totalvirusfound = 0;
    totalfilesscanned = 0;
    totalwarnings = 0;
    infoText->clear();
    infoText->append( "Start scanning..." );
    socket->connectToHost(ClamAV_host, clamav_main_port );
    sendStringToServer("STREAM");
    	
    }
    
    void sendStringToServer(const QString &stringtosend)
    {
	// write to the server
	Q3TextStream os(socket);
	os << stringtosend << "\n";
    }

    void sendFileToServer(const QString &file_to_scan)
    {
	char buffer[1500];
	int nbytes;
	int file_size;
	int stream_percent;

	#ifdef debug
	sprintf (&Output_String[0], "Scanning...");
       	infoText->append( Output_String );
	#endif
        infoText->append(file_to_scan );	
	totalfilesscanned++;
	QFile file(file_to_scan);
	if (file.open (QIODevice::ReadOnly))
	{
	file_size = (int)file.size();
	#ifdef debug
	sprintf (&Output_String[0], "File size is %d", file_size);
	infoText->append( Output_String );
	#endif
	stream_percent = 0;
	streamprogress->setTotalSteps (file_size);
	while (((nbytes = file.readBlock(buffer, sizeof(buffer))) > 0) && (file_size <= max_stream_size)) 
		{
		stream_percent += nbytes;
		if (stream_socket->isOpen())
		{
		if (stream_socket->writeBlock(buffer, sizeof(buffer)) == -1)
		   {		   
		   infoText->append("Unexpected Error: writeBlock error");	
		   }
		streamprogress->setProgress (stream_percent );
		}
		else
		   infoText->append("Unexpected Error: stream_socket is closed");	
		}		
        if (file_size > max_stream_size)
	   {
	   totalwarnings++;
	   infoText->setColor(QColor(192,0,192));			
	   infoText->append("Warning: the above file is too large and could not be scanned (max_stream_size).");	
	   infoText->setColor(QColor(0,0,0));			
	   }
	file.close();	
	}
	if (stream_socket->state() == Q3Socket::Connected)
		stream_socket->close(); //close socket to perform scan
	else
	   infoText->append("Unexpected Error: the STREAM socket has been already closed!");	
    }
    
    
    void socketReadyRead()
    {
	// read from the server
	while ( socket->canReadLine() ) {
	    Socket_Read_Line = socket->readLine();
	    infoText->setColor(QColor(0,0,0));
	    if (strstr(Socket_Read_Line, "PORT") != NULL)
	        {
		if (sscanf(Socket_Read_Line, "PORT %hu\n", &clamav_stream_port) != 1)
		    {
		    #ifdef debug
		    infoText->append( "Can't get PORT information" );
		    #endif
		    clamav_stream_port = 0;
		    }
	        else
		    {
		    // connect to the STREAM server
		    #ifdef debug
		    sprintf (&Output_String[0], "Opening STREAM port %hu....", clamav_stream_port);
		    infoText->append( Output_String );
		    #endif
		    stream_socket->connectToHost( ClamAV_host, clamav_stream_port );		   
		    }
		#ifdef debug
	        infoText->append( Socket_Read_Line );
		#endif
		}
	    else if (strstr(Socket_Read_Line, "FOUND") != NULL)
		{
		 //Clamd --> A VIRUS FOUND 
		totalvirusfound++;
		infoText->setColor(QColor(255,0,0));
	        infoText->append( Socket_Read_Line );
		scan_next_file();
		}
	    else if (strstr(Socket_Read_Line, "OK") != NULL)
		{
		 //Clamd --> NO VIRUS FOUND
		#ifdef debug
		infoText->setColor(QColor(0,192,0));
	        infoText->append( Socket_Read_Line);
		#endif
		scan_next_file();
		}
	    else if (strstr(Socket_Read_Line, "ClamAV") != NULL)
		{
		infoText->setColor(QColor(0,0,255));
	        infoText->append( Socket_Read_Line );
		}
	    else
	    	{
	    	infoText->setColor(QColor(0,0,0));
		#ifdef debug
		infoText->append( Socket_Read_Line );
		#endif
		}
	    	
	    //restore standard color
	    infoText->setColor(QColor(0,0,0));
	}
    }

    void socketConnected()
    {
	#ifdef debug
	infoText->append( "Connected to ClamAV server" );
	#endif
    }
    
    void Stream_socketConnected()
    {
   	if (it != files_to_scan.end() )
    	{	
    	FileToScan = *it;				       
    	if ( !FileToScan.isEmpty() )
    		{
		sendFileToServer(FileToScan);
		}
	}
	else
		{
		scanninginprogress = 0;
		infoText->append( "Scan terminated" );
		}
    }
    
    void Stream_socketConnectionClosed()
    {
    //STREAM has been closed on end of file to start scanning it
    }
    
    void socketConnectionClosed()
    {
   	#ifdef debug
	infoText->append( "Connection closed by the ClamAV server" );
	#endif
	if (scanninginprogress == 1)
		{
		#ifdef debug
		infoText->append( "Opening new STREM..." );
		#endif
		socket->connectToHost(ClamAV_host, clamav_main_port );
		sendStringToServer("STREAM");
		}
    }


    void socketError( int e )
    {
	infoText->setColor(QColor(255,0,0));
	infoText->append( tr("Error number %1 occurred\n").arg(e) );
	infoText->setColor(QColor(0,0,0));

    }
    
    void scan_next_file()
    {
     streamprogress->setProgress ( 0 );
     if (scanninginprogress == 1)
	{
	++it;    
	if (it == files_to_scan.end() )
		{
		if (totalvirusfound > 0)
			{
			infoText->setColor(QColor(255,0,0));			
			sprintf (&Output_String[0], "*** ClamAV found (%d) virus(es) - %d file(s) scanned ***", totalvirusfound, totalfilesscanned );
			infoText->append( Output_String );
			infoText->setColor(QColor(0,0,0));			
			}
		else
			{
			infoText->setColor(QColor(0,192,0));			
			sprintf (&Output_String[0], "*** No virus found - %d file(s) scanned ***", totalfilesscanned );
			infoText->append( Output_String );
			infoText->setColor(QColor(0,0,0));
			}
		if (totalwarnings > 0)
			{
			infoText->setColor(QColor(192,0,192));			
			sprintf (&Output_String[0], "*** Total warning(s) %d ***", totalwarnings );
			infoText->append( Output_String );
			infoText->setColor(QColor(0,0,0));
			}
		totalwarnings = 0;
		totalfilesscanned = 0;
		totalvirusfound = 0;	    
		scanninginprogress = 0;
		}
	}
     }


private:
    int totalfilesscanned;
    int totalvirusfound;
    int totalwarnings;
    int scanninginprogress;
    QString ClamAV_host;
    QString FileToScan;
    QStringList files_to_scan;
    QStringList::Iterator it;
    Q3Socket *socket;
    Q3Socket *stream_socket;
    Q3TextView *infoText;
    QLineEdit *inputText;
    QString Socket_Read_Line;
    Q3ProgressBar *streamprogress;
    char Output_String[512];
    const QString host;
    Q_UINT16 clamav_main_port;
    Q_UINT16 clamav_stream_port;

};


int main( int argc, char** argv )
{
    QApplication app( argc, argv );
    Client QtClamAVclient( argc<2 ? "localhost" : argv[1], argc<3 ? "3310" : argv[2]);
    app.setMainWidget( &QtClamAVclient );
    QtClamAVclient.setCaption(QTCLAMAVVER);
    QtClamAVclient.resize(500,240);
    QtClamAVclient.show();
    return app.exec();
}

#include "qtclamavclient.moc"

